package main

import (
	"context"
	"fmt"
	"io"
	"log"
	"math/rand"
	"net/http"
	"os"
	"strings"
	"text/template"

	"github.com/minio/minio-go/v7"
	"github.com/minio/minio-go/v7/pkg/credentials"
	"gopkg.in/yaml.v3"
)

func GetPicsArray() []string {
	files, err := os.ReadDir("images")
	if err != nil {
		panic(err)
	}
	if len(files) == 0 {
		return []string{"no images"}
	}
	var filesNames = make([]string, len(files))
	for i, file := range files {
		filesNames[i] = string("images/" + file.Name())
	}
	return filesNames
}

func GetRandImg(conf Config) string {

	var picsArray []string

	if conf.UseS3 == true {
		picsArray = GetObjectsFilenames(conf)
	} else {
		picsArray = GetPicsArray()
	}

	return picsArray[rand.Intn(len(picsArray))]
}

func (conf *ConfHandler) ReturnPicFilename(w http.ResponseWriter, req *http.Request) {
	var picFilename io.Reader = strings.NewReader(GetRandImg(conf.ConfigHandler))
	log.Println("Картинка отдана")
	io.Copy(w, picFilename)
}

/*func ReturnPic(w http.ResponseWriter, req *http.Request) {
	picture, _ := os.Open(fmt.Sprintf("images/%v", GetRandImg()))
	var pictureReader io.Reader = picture
	io.Copy(w, pictureReader)
	log.Println("Картинка отдана")
}*/

func (conf *ConfHandler) ReturnHTML(w http.ResponseWriter, req *http.Request) {
	htmlOpen, err := template.ParseFiles("dist/index.html")
	if err != nil {
		log.Println(err.Error())
		return
	}
	image := GetRandImg(conf.ConfigHandler)
	fmt.Println(image)
	htmlOpen.Execute(w, image)
}

type Config struct {
	UseS3     bool   `yaml:"use_s3"`
	Address   string `yaml:"address"`
	Bucket    string `yaml:"bucket"`
	AccessKey string `yaml:"access_key"`
	SecretKey string `yaml:"secret_key"`
}

type ConfHandler struct {
	ConfigHandler Config
}

func InitConf(filename string) Config {
	var conf Config
	configFile, _ := os.ReadFile(filename)
	err := yaml.Unmarshal(configFile, &conf)
	if err != nil {
		log.Fatal(err)
	}

	return conf
}

func GetObjectsFilenames(conf Config) []string {
	minioClient, err := minio.New(conf.Address, &minio.Options{
		Creds:  credentials.NewStaticV4(conf.AccessKey, conf.SecretKey, ""),
		Secure: true,
	})

	if err != nil {
		log.Fatalln(err)
	}

	ctx, cancel := context.WithCancel(context.Background())

	defer cancel()

	objectCh := minioClient.ListObjects(ctx, conf.Bucket, minio.ListObjectsOptions{Recursive: false})

	fmt.Println(objectCh)
	objectsKeyArr := []string{}

	for object := range objectCh {
		if object.Err != nil {
			fmt.Println(object.Err)
		}
		objectsKeyArr = append(objectsKeyArr, "https://"+conf.Address+"/"+conf.Bucket+"/"+object.Key)
	}
	return objectsKeyArr
}

func main() {
	log.Println("Server Start")
	conf := InitConf("config.yaml")
	confHandler := ConfHandler{ConfigHandler: conf}
	imagesDir := http.FileServer(http.Dir("images"))
	staticDir := http.FileServer(http.Dir("dist/assets"))
	http.Handle("/images/", http.StripPrefix("/images/", imagesDir))
	http.Handle("/assets/", http.StripPrefix("/assets/", staticDir))

	http.HandleFunc("/rand", confHandler.ReturnPicFilename)
	http.HandleFunc("/", confHandler.ReturnHTML)
	http.ListenAndServe(":3666", nil)
}
